#ifndef BitVecIncluded
#define BitVecIncluded

// copyright (c) 1992 by Paul Wheaton
// 1916 Brooks #205, Missoula, MT  59801
//
// voice phone:  (406)543-7543
// modem phone:  (406)543-1144 (2400N81)
//  CompuServe:  72707,207
//    Internet:  72707.207@CompuServe.com

#include <Strings.h>
#include <ByteVec.h>

// for access by BitVector only
class BitRef
  {
      Byte huge* P;  // pointer to byte containing bit
      Byte M;   // mask for bit
      BitRef(Byte huge* BP, Long BitOffset); // non-inlined due to 16M stuff
      friend BitVector;
    public:
      operator Bool()         { return ((*P & M) != 0); }
      Bool operator=(Bool b)
        {
          if (b) *P |= M;
          else *P &= ~M;
          return b;
        }
      void operator&=(Bool b) { if (!b) *P &= ~M; }
      void operator|=(Bool b) { if (b) *P |= M; }
      void operator^=(Bool b) { if (b) *P ^= M; }
  };

class BitVector : public ByteVector
  {
      Long BitLen; // the number of bits used
      void CtorsHelper(const Long* LP,Long Length);
    public:
      //  see the notes on the constructors at the end of this file
      BitVector();
      BitVector(long);
      BitVector(long,long);
      BitVector(long,long,long);
      BitVector(long,long,long,long);
      BitVector(long,long,long,long,long);
      BitVector(long,long,long,long,long,long);
      BitVector(long,long,long,long,long,long,long);
      BitVector(long,long,long,long,long,long,long,long);
      BitVector(long,long,long,long,long,long,long,long,long);
      BitVector(const Byte* B, Long Len); // direct copy
      BitVector(const BitVector&);
      BitRef Ref(Long Index);
      BitRef operator[](Long Index);
        /* used for setting or retrieving bit elements.  Accessing beyond the
           last bit will make this vector longer. */
      Bool operator()(Long Index) const {return At(Index);}
        /* used for reading bits only.  Accessing beyond the last bit will
           return a 0 (False).  This is much faster than [] */
      Bool At(Long Index) const;
      BitVector operator()(Long Index,Long Len) const {return At(Index,Len);}
        // for a subset
      BitVector At(Long Index, Long Len) const;
      BitVector Before(Long Index) const {return At(0,Index);}
      BitVector Through(Long Index) const {return At(0,Index+1);}
      BitVector From(Long Index) const;
      BitVector After(Long Index) const;
      void operator=(const BitVector&);
      BitVector operator!() const;
      BitVector operator&(const BitVector&) const;
      BitVector operator^(const BitVector&) const;
      BitVector operator|(const BitVector&) const;
      void operator&=(const BitVector&);
      void operator^=(const BitVector&);
      void operator|=(const BitVector&);
      void ReAlloc(Long NewCapacity);
      Long Size() const {return BitLen;}
      Long ByteSize() const {return Len;}
      void ClearRange(Long Index1,Long Index2);
      void SetRange(Long Index1,Long Index2,Bool X=On);
      operator String() const;
  };

inline Long BytesNeeded(Long NumBits) {return(((NumBits-1)/8)+1);}

#endif

/*

BitVector constructors:

  I wanted to use a variable argument constructor (see your favorite texts
  on "elipsis", "...", "variable arguments" or "va_arg, va_end and
  va_start") but there were limitations that I felt could lead to very hard
  to find bugs, namely that the applications programmer must first pass a
  value that reveals how many parameters there are.  Being off by one could
  lead to a variety of problems.

  There are four ways to construct a BitVector:

    1) BitVector BV;

       This creates a vector that will have a length of 0 although some
       memory will be allocated so that elements may be added later (which
       is the most likely thing to happen).

    2) BitVector BV(1,2,55,8);

       This creates a bit vector of length 56 (7 bytes long minimum).  All
       of the bits are 0 (False) except for bits 1, 2, 55 and 8 which are 1
       (True).  You may pass 1 to 9 parameters using this type of
       constructors.  You're limited to 9 because the Glock C++ name
       mangling mixed with MSC 5.1 limits it this way (xxx).  An update of
       either of these may solve this problem.

    3) static Byte BA[]={1,2,55,8};
       BitVector BV(BA,4);

       This creates a vector of length 4, copying the elements from BA.
       You don't have to copy from an array of bytes, you can use any kind
       of array or even a pointer to anywhere in memory.

    4) BitVector BV2(BV);

       Create a BitVector from another BitVector.

*/
