#ifndef ByteVecIncluded
#define ByteVecIncluded

// copyright (c) 1992 by Paul Wheaton
// 1916 Brooks #205, Missoula, MT  59801
//
// voice phone:  (406)543-7543
// modem phone:  (406)543-1144 (2400N81)
//  CompuServe:  72707,207
//    Internet:  72707.207@CompuServe.com

/* things to add
     lower bounds (in an inherited class)
     Minimize() member function to make Alloc==Len
     virtual memory handler (using disk)
     ExtraAlloc() mem func to set how much extra is to be allocated each
       time more memory is allocated (this will speed up rapidly growing
       vectors)
     vector sizes beyond 64k
*/

#include <Misc.h>

class File;

typedef const void* ConstVoidPointerType;  // xxx used to clear up cfront prob

class ByteVector
  {
      Byte* P; // pointer to first Byte in storage
      Long Len;   // Length of vector
      Long Alloc; // amount of actual storage allocated
      Long Extra; // amount of extra memory to allocate when getting more memory
      void ReNew(Long NewCapacity);
      Byte huge & Ref(Long Index);
      friend class BitVector;
      void Assign(const ByteVector& BV);
      void FreeMemory();  // equivalent to new or delete
      Byte Selectors(); // 16M only:  number of selectors used.  0 if using malloc
      friend void DebugDisp(const ByteVector& BV);
      friend class File;
      friend class RecFile;
      friend class ObjVec;
    public:
      ByteVector();
      ByteVector(int);          // although these say "int", they'll be
      ByteVector(int,int);      // converted to Bytes.  "int" makes for shorter
      ByteVector(int,int,int);  // mangled identifiers than "Byte"
      ByteVector(int,int,int,int);
      ByteVector(int,int,int,int,int);
      ByteVector(int,int,int,int,int,int);
      ByteVector(int,int,int,int,int,int,int);
      ByteVector(int,int,int,int,int,int,int,int);
      ByteVector(int,int,int,int,int,int,int,int,int);
      //  see the notes on the constructors at the end of this file
      ByteVector(void* B, Long Len);  // copy from some pointer
      ByteVector(const ByteVector&);
      ByteVector(File& F);  // read a ByteVector object from a file
      ~ByteVector(){FreeMemory();}
      void ExtraAlloc(Long Quan){Extra=Quan;}
        // each time more memory is needed, this is how much extra is allocated
      Byte huge & operator[](Long Index){return Ref(Index);}
        //  To access and modify specific bytes of the vector.
        //  Accessing beyond the current length will increase the vector size.
        //  Completely range protected.
      void operator=(const ByteVector& BV) {Assign(BV);}
      Bool operator==(const ByteVector& S);
      Bool operator!=(const ByteVector& S) { return !(*this==S); }
      // vector concatenation
      ByteVector operator+(const ByteVector& B);
      ByteVector operator+(Byte B);
      friend ByteVector operator+(Byte B, const ByteVector& BV);
      void operator+=(const ByteVector&);
      void operator+=(Byte B) {Ref(Len)=B;}
      operator ConstVoidPointerType(){return P;}
      void CopyTo(void* Dest, Long Bytes=MaxLong); // copy from vector to memory
      void CopyFrom(void* Source, Long Bytes); // copy from memory to vector
      //friend void operator=(void* Dest,const ByteVector& BV) {BV.CopyTo(Dest);}
      void WriteTo(File& F);  // write a ByteVector object to file
      Long Capacity() {return Alloc;}
      Long ReAlloc(Long NewCapacity);
      void MinimizeMemory();
        // reduce memory consumption to just what is needed now
      Long Size() {return Len;}
      Long Index(Byte SearchByte, Long StartIndex=0);
        // returns the const NotFound if the Byte is not found in the vector
        // otherwise returns the index

      Byte   operator()(Long Index); // return an indexed byte.
      ByteVector operator()(Long Index, Long Length); // return a subvector
      Byte   At(Long I);
      ByteVector At(Long Index, Long Length);
      ByteVector Before(Long Index) {return At(0,Index);}
      ByteVector Through(Long Index) {return At(0,Index+1);}
      ByteVector From(Long Index);
      ByteVector After(Long Index);
      void Insert(Byte C,Long Index=0);
      void Insert(const ByteVector& BV,Long Index=0);
      void Delete(Long Index=0,Long Length=1);
      void Clear(Byte B=0); // set all elements to B
      Long Sum();
      void Empty(){Len=0;}
  };

extern Bool BigVectorJump;
/* If true, then vectors in size >64k will always be allocated in chunks of
64k.  This saves on fragmentation and recopying time. Default is True. */

const DefaultVectorExtra=16;

#endif

/*

ByteVector constructors:

  I wanted to use a variable argument constructor (see your favorite texts
  on "elipsis", "...", "variable arguments" or "va_arg, va_end and
  va_start") but there were limitations that I felt could lead to very hard
  to find bugs, namely that the applications programmer must first pass a
  value that reveals how many parameters there are.  Being off by one could
  lead to a variety of problems.

  There are four ways to construct a ByteVector:

    1) ByteVector BV;

       This creates a vector that will have a length of 0 although some
       memory will be allocated so that elements may be added later (which
       is the most likely thing to happen).

    2) ByteVector BV(1,2,55,8);

       This creates a vector of length 4.  You may pass 1 to 9 parameters
       using this type of constructors.  You're limited to 9 because the
       Glock C++ name mangling mixed with MSC 5.1 limits it this way.  An
       update of either of these may solve this problem.

    3) static Byte BA[]={1,2,55,8};
       ByteVector BV(BA,4);

       This creates a vector of length 4, copying the elements from BA.
       You don't have to copy from an array of bytes, you can use any kind
       of array or even a pointer to anywhere in memory.

    4) ByteVector BV2(BV);

       Create a ByteVector from another ByteVector.

*/

