/* mark_stack() reports the amount of unused stack space
   over the duration of a programs execution;
   useful for adjusting /STACK smaller to save memory 
   -- for Microsoft C 
 */

#include <stdio.h>
#include <malloc.h>

extern char end;                        /* stack bottom in crt0.asm */

static char * stack_bot = (char *) &end;
static char mark = 0xff;                /* clean stack footprint */
static char * ptr;

static void report_stack(void)          /* atexit style function */
{
    static unsigned unused;
    unused = 0;
    ptr = stack_bot;
    while (*ptr++ == mark) 
        unused++;
    printf("\n\nUnused Stack = %u Bytes\n\n",unused);
}

void mark_stack(void)
{
    static char * stack_top;
    stack_top = stack_bot + stackavail();   /* OK within 256 bytes */
    ptr = stack_bot;
    do 
    {
        *ptr++ = mark;
    } 
        while ( ptr < stack_top);
    atexit(report_stack);               /* log function ptr with atexit */
}

#ifdef NDEBUG                           /* conditional compile */
int main (void)
{
    mark_stack();                       /* ENTER ONCE AS 1ST STMT    */
    exit(0);                            /* report_stack() upon exit  */
}
#endif