/****************************************************************
 * File: strvcat.c
 * Author: John Rex
 * Purpose: implements strvcat(), a variant of strcat that will
 *          concatenate multiple strings together
 *
 * Usage:
 *   Prototype: char * strvcat(char *target, ...);
 *
 *   Usage:     strvcat(buf, s1, s2, s3, NULL);
 *
 *     would be equivalent to
 *
 *              strcat(buf, s1);
 *              strcat(buf, s2);
 *              strcat(buf, s3);
 *
 *   The last argument to strvcat() must be NULL.
 *
 * Compile time switches:
 *   TEST - brings in a test driver
 *
 * Compilers: Microsoft C 5.1
 *            Turbo C 2.0
 *
 * Memory Model: any
 ***********************************************************************/

#include <stdarg.h>
#include <stdio.h>
#include <string.h>

char *strvcat(char *target, ...)
{
    va_list marker;
    char *s;

    va_start(marker, target);           /* set up to retrieve args */

    /* loop through them */
    s = va_arg(marker, char *);         /* get first argument */
    while (s != NULL) {
        strcat(target, s);              /* concatenate it */
        s = va_arg(marker, char *);     /* get next argument */
    }

    va_end(marker);     /* clean up */
    return (target);    /* that's all! */
}

#if defined TEST
int main() { char buffer[80];

    strcpy(buffer, "Four");
    strvcat(buffer, " score", " and", " seven", " years", NULL);
    printf("%s\n", buffer);

    strvcat(buffer, NULL);
    printf("%s\n", buffer);

    strvcat(buffer, " ago", NULL);
    printf("%s\n", buffer);
}
#endif

